package cz.muni.fi.pb138.dtdToXsd;

import java.io.InputStream;
import java.io.File;
import java.util.List;

/**
 * DTDCuuter Interface
 * 
 * @author Roman Jakubco 374241
 */
public interface DTDCutter {
    
     /**
      *  Method for parsing given InputStream(DTD) and using methods for getting
      *  data from it and storing it in Collections
      *  
      * @param sourceDTD is given stream for DTD file
      */ 
     public void parseDTD(InputStream sourceDTD);
     
     /**
     * Method for parsing given String which contains all information about element
     * from DTD file. Method also format this String. Unimportant parentheses are
     * deleted also line separators, blank spaces and tabulators.
     * 
     *
     * @param  elementString given String containing whole ELEMENT tag
     * @return ElementDTD contains parsed data from given String
     */
    public ElementDTD getElementFromSource(String elementString);

    /**
     * Method for parsing given String which contains all information about
     * attribute from DTD file
     * 
     *
     * @param  attributeString given String containing whole ATTLIST tag
     * @return Attribute contains parsed data from given String 
     */
    public Attribute getAttributeFromSource(String attributeString);

    /**
     * Method for parsing given String which contains all information about
     * entity from DTD file
     *
     * @param entityString given String containing whole Entity tag
     * @return Entity contains parsed data from given String
     */
    public Entity getEntityFromSource(String entityString);
    

    /**
     * Method returning collection of attributes from parsed DTD
     * @return unmodifiable list of attributes from DTD
     */
    public List<Attribute> getAttributes() ;     
    
    /**
     * Method returning collection of elements from parsed DTD
     * @return unmodifiable list of elements from DTD
     */
    public List<ElementDTD> getElements(); 
        
    /**
     * Method returning collection of entities from parsed DTD
     * @return unmodifiable list of entities from DTD
     */
    public List<Entity> getEntities() ;
    
    /**
     * Method returning collection of comments from parsed DTD
     * @return unmodifiable list of comments from DTD
     */
    public List<String> getComments();
    
    /**
     * Method parses DTD and checks for more than one attribute in ATTLIST
     * tag. If that tag exists then method creates ATTLIST tag for each one attribute
     * and this formated output is stored in new DTD file, which is then returned     
     * 
     * @param inputStream
     * @return File file which contains all DTD tags in default format
     */
    public File prepareFile(InputStream inputStream) throws XSDConvertException;
    
    /**
     * Method for parsing XML and looking for DTD which can be in XML.
     * If there are DTD tags then method check if there are more than one attribute
     * in one ATTLIST tag if so then it is changed to default format (only one
     * attribute in one ATTLIST tag). Then every line with DTD tag is copied to new
     * File.
     * 
     * 
     * @param inputStream
     * @return File file which contains only DTD tags from XML in default format
     */
    public File prepareInternFile(InputStream inputStream) throws XSDConvertException;
    



    
}
