package cz.muni.fi.pb138.dtdToXsd;

import java.io.*;
import java.util.*;

/**
 * Author: Roman Jakubco 374241 Date: 20.4.2012
 *
 */
public class DTDCutterImpl implements DTDCutter {

    private List<ElementDTD> elements = new ArrayList<>();
    private List<Attribute> attributes = new ArrayList<>();
    private List<Entity> entities = new ArrayList<>();
    private List<String> comments = new ArrayList<>();

    @Override
    public void parseDTD(InputStream sourceDTD) {
        String temp;
        Scanner scanner = new Scanner(sourceDTD).useDelimiter("<!");
        while (scanner.hasNext()) {
            temp = scanner.next();
            //finding comments
            if (temp.startsWith("--")) {
                temp = temp.replaceAll("\\s+", " ");
                temp = temp.trim();
                String comment = temp.substring(2, temp.length() - 3);
                comments.add(comment);
                continue;
            }
            temp = temp.replace(System.getProperty("line.separator"), "");
            if (temp.indexOf(">") != temp.lastIndexOf(">")) {
                temp = temp.substring(0, temp.indexOf(">") + 1);
            }
            //finding elements
            if (temp.contains("ELEMENT")) {
                elements.add(getElementFromSource(temp));
                continue;
            }
            //fidning attributes
            if (temp.contains("ATTLIST")) {
                attributes.add(getAttributeFromSource(temp));
                continue;
            }
            //finding entities
            if (temp.contains("ENTITY")) {
                entities.add(getEntityFromSource(temp));
            }
        }
    }

    @Override
    public ElementDTD getElementFromSource(String source) {
        ElementDTD elementDTD = new ElementDTD();
        source = source.replaceAll("\\s+", " ");
        String[] split = source.split(" ", 3);
        elementDTD.setElementName(split[1].trim());
        //odstranenie odriadkovania a odstranenie medzier na konci a na zaciatku
        split[2] = split[2].replace(System.getProperty("line.separator"), "");
        split[2] = split[2].trim();
        //pouzitie Scanner.. uzitocna trieda na parsovanie i ked trosku na rozmyslanie 
        split[2] = split[2].substring(0, split[2].length() - 1);
        split[2] = split[2].replace(" ", "");
        Scanner scanner = new Scanner(split[2]);

        //regularny vyraz, ktory hlada vsetky slova moznosti
        //slov ktore su uzatvorkovane a nie je v nich ',' alebo '|' 
        String regex = "[\\(]+[[\\w[\\p{Punct}]&&[^()]]&&[^,|]]+[\\)?*+]+";
        String temp;
        String help;

        //cyklus cez string ktory hlada vzdy taky vyraz a odstranuje z neho zatvorky
        while ((temp = scanner.findInLine(regex)) != null) {
            help = count(temp);
            split[2] = split[2].replace(temp, help);
        }
        elementDTD.setCategory(split[2]);
        return elementDTD;
    }

    @Override
    public Attribute getAttributeFromSource(String source) {
        Attribute attribute = new Attribute();
        //odstranenie nadbztocnych medzier kde je ich viac ako jedna
        source = source.replaceAll("\\s+", " ");
        source = source.trim();
        source = source.substring(0, source.length() - 1);
        source = source.trim();
        String[] splitString = source.split(" ", 4);

        attribute.setElementName(splitString[1]);
        attribute.setName(splitString[2]);

        // naivne osetrenie a rozdelenie zvysku v Stringu podla moznosti
        //<!ATTLIST Administrator       Extension  CDATA #IMPLIED>
        // <!ATTLIST DatabaseAttributes  Type       (Production | Development | Testing) #REQUIRED>
        //jedine moznosti preto zistenie ci je tam ')' ak nie tak normalne cez medzeru
        String[] tempSplit = splitString[3].split("\\)");
        if (tempSplit.length == 2) {
            tempSplit[0] = tempSplit[0].replaceAll("\\s+", "");
            tempSplit[0] = tempSplit[0].concat(")");

        } else {
            tempSplit = splitString[3].split(" ", 2);

        }
        if (tempSplit.length == 1) {
            attribute.setType(tempSplit[0]);
            attribute.setDefaultType("#IMPLIED");
            return attribute;
        }
        attribute.setType(tempSplit[0]);

        //tu by trebalo osetrit ked je FIXED+ defualt type  specialnz pripad
        //dohodnut sa naprogramovat neskor
        tempSplit[1] = tempSplit[1].trim();
        String[] splitDefaultType = tempSplit[1].split(" ", 2);

        if (splitDefaultType.length == 2) {
            splitDefaultType[0] = splitDefaultType[0].trim();

            attribute.setDefaultType(splitDefaultType[0]);
            splitDefaultType[1] = splitDefaultType[1].substring(1, splitDefaultType[1].length() - 1);
            attribute.setFixed(splitDefaultType[1]);
        } else {
            tempSplit[1] = tempSplit[1].trim();
            if (tempSplit[1].contains("\"")) {
                tempSplit[1] = tempSplit[1].substring(1, tempSplit[1].length() - 1);
            }

            attribute.setDefaultType(tempSplit[1]);
        }

        return attribute;
    }

    @Override
    public Entity getEntityFromSource(String source) {
        // probem ohladne entity pozeram ze ona moze mat ked tak az 3 hodnoty.. toto je len pre jednoduchu
        //urcite v buducnosti zmena ked vyriesime ako na to
        source = source.replaceAll("\\s+", " ");
        Entity entity = new Entity();
        String[] split = source.split(" ", 3);
        entity.setEntityName(split[1].trim());
        split[2] = split[2].replace(System.getProperty("line.separator"), "");
        split[2] = split[2].substring(1, split[2].length() - 2);
        entity.setValue(split[2].trim());
        return entity;
    }

    @Override
    public List<Attribute> getAttributes() {
        return Collections.unmodifiableList(attributes);
    }

    @Override
    public List<ElementDTD> getElements() {
        List<ElementDTD> temp = new ArrayList<>();
        List<ElementDTD> tempAll = new ArrayList<>();
        List<ElementDTD> skuska = new ArrayList<>();
        Set<ElementDTD> vysledok = new HashSet<>();
        tempAll.addAll(elements);
        //presunutie vsetkych elementov, ktore neodkazuju na iny element
        for (ElementDTD elementDTD : elements) {
            if ((elementDTD.getCategory()).equalsIgnoreCase("#PCDATA") || (elementDTD.getCategory()).equalsIgnoreCase("ANY")
                    || (elementDTD.getCategory()).equalsIgnoreCase("EMPTY")) {
                tempAll.remove(elementDTD);
                temp.add(elementDTD);
                continue;
            }
        }
        //zoradenie elementov podla toho ako chceme
        //teda ze najpv su elementy ktore referencuju ine elementy
        // a az za nimi su tie elementz atd 
        //zoradia sa uplne naopak takze po cykle len reverse
        skuska.addAll(tempAll);
        for (ElementDTD elementDTD : tempAll) {
            for (ElementDTD elecat : skuska) {
                if (elementDTD.equals(elecat)) {
                    continue;
                }
                if ((elecat.getCategory().contains(elementDTD.getElementName()))) {
                    vysledok.add(elecat);
                }
            }
            vysledok.add(elementDTD);
        }

        // spojenych zoradenych elementov spolu s tymi, ktore na nic neodkazuju
        elements.clear();
        elements.addAll(vysledok);
        //reverse collection kedze ten moj cyklus ich zoraduje naopak toto mi pride
        //lahsie ako ho upravovat
        Collections.reverse(elements);
        elements.addAll(elements.size(), temp);

        return Collections.unmodifiableList(elements);
    }

    @Override
    public List<Entity> getEntities() {
        return Collections.unmodifiableList(entities);
    }

    @Override
    public List<String> getComments() {
        return Collections.unmodifiableList(comments);
    }

    @Override
    public File prepareFile(InputStream inputStream) throws XSDConvertException {
        BufferedReader br = null;
        BufferedWriter bw = null;
        File file = new File("test.dtd");
        try {
            br = new BufferedReader(new InputStreamReader(inputStream));
            bw = new BufferedWriter(new FileWriter(file));
            String line;
            String[] temp;

            while ((line = br.readLine()) != null) {

                line = line.trim();
                line = line.replaceAll("\\s+", " ");

                //there is more than one attribute in one ATTLIST tag                
                if (line.indexOf("<!ATTLIST") == 0) {
                    if (line.lastIndexOf(">") == line.length() - 1) {
                        bw.write(line);
                        bw.newLine();
                    } else {
                        //pripad ked je je ich viac v jenom tagu a su podelene po riadkoch
                        temp = line.split(" ", 5);
                        line = line.concat(">");
                        bw.write(line);
                        bw.newLine();
                        //read new line until '>' is not found
                        //creating new '<!ATTLIST' tags for every attribute 
                        while (!((line = br.readLine().trim()).contains(">"))) {
                            //if there is empty line just skip
                            if (line.indexOf(System.getProperty("line.separator")) == 0) {
                                continue;
                            }
                            //concat ending tag to <!ATTLIST nameOfElement 
                            line = line.replace(">", "");
                            String concatString = temp[0] + " " + temp[1] + " " + line + ">";
                            bw.write(concatString);
                            bw.newLine();
                        }
                        String concatString = temp[0] + " " + temp[1] + " " + line;
                        bw.write(concatString);
                        bw.newLine();
                    }
                } else {
                    bw.write(line);
                    bw.newLine();
                }
                if ((line.endsWith("]>")) || (line.endsWith("] >"))) {
                    break;
                }
            }
        } catch (FileNotFoundException ex) {
            throw new XSDConvertException(ex);
        } catch (IOException ex) {
            throw new XSDConvertException(ex);
        } finally {
            try {
                if (bw != null) {
                    bw.close();
                }
            } catch (IOException ex) {
                throw new XSDConvertException(ex);
            }
        }
        return file;
    }

    public File prepareInternFile(InputStream inputStream) throws XSDConvertException {
        BufferedReader br = null;
        BufferedWriter bw = null;
        File file = new File("testXML.dtd");
        try {
            bw = new BufferedWriter(new FileWriter(file));
            Scanner scanner = new Scanner(inputStream);
            //finding [ in XML because only there can be DTD information
            //and testing if there is. If not there file with text "noDTD" is returned
            String test = scanner.findWithinHorizon("\\[", 0);
            if (test == null) {
                bw.write("no DTD");
                bw.newLine();
                return file;
            }

            String line;
            String[] temp;
            scanner.useDelimiter("<!");
            while (scanner.hasNext()) {
                line = scanner.nextLine();
                line = line.trim();
                line = line.replaceAll("\\s+", " ");
                if(line.isEmpty()){
                    continue;
                }
                //Finding only tags of attributes               
                if (line.indexOf("<!ATTLIST") == 0) {
                    if (line.lastIndexOf(">") == line.length() - 1) {                        
                        bw.write(line);
                        bw.newLine();
                    } else {
                        //there is more than one attribute in one ATTLIST tag 
                        temp = line.split(" ", 5);
                        line = line.concat(">");
                        
                        bw.write(line);
                        bw.newLine();
                        //read new line until '>' is not found
                        //creating new '<!ATTLIST' tags for every attribute 
                        while (!(line=scanner.nextLine()).contains(">")) {
                            //if there is empty line just skip
                            if (line.indexOf(System.getProperty("line.separator")) == 0) {
                                continue;
                            }
                            //concat ending tag to <!ATTLIST nameOfElement 
                            line = line.replace(">", "");
                            String concatString = temp[0] + " " + temp[1] + " " + line + ">";
                            bw.write(concatString);
                            bw.newLine();
                        }
                        String concatString = temp[0] + " " + temp[1] + " " + line;
                        bw.write(concatString);
                        bw.newLine();
                    }
                } else {                    
                    bw.write(line);
                    bw.newLine();
                }
                if ((line.endsWith("]>")) || (line.endsWith("] >"))) {
                    break;
                }
            }
        } catch (FileNotFoundException ex) {
            throw new XSDConvertException(ex);
        } catch (IOException ex) {
            throw new XSDConvertException(ex);
        } finally {
            try {
                if (bw != null) {
                    bw.close();
                }
            } catch (IOException ex) {
                throw new XSDConvertException(ex);
            }
        }
        return file;
    }

    //only helping method for counting parentheses in given string and deleting 
    //them
    public String count(String input) {
        int left = input.split("\\Q" + "(" + "\\E", -1).length - 1;
        int right = input.split("\\Q" + ")" + "\\E", -1).length - 1;

        if (left == right) {
            input = input.replace("(", "").replace(")", "");
            return input;
        }
        if (left > right) {
            input = input.replace("(", "").replace(")", "");
            int diff = left - right;
            for (int i = 1; i <= diff; i++) {
                input = "(" + input;
            }
        } else {
            input = input.replace("(", "");

            int help = 0;
            for (int i = 1; i <= left; i++) {
                help = input.indexOf(")", help);
                input = input.substring(0, help).concat(input.substring(help + 1, input.length()));
            }
        }
        return input;
    }
}
