package cz.muni.fi.pb138.dtdToXsd;

import java.io.*;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.logging.Level;
import javax.xml.parsers.ParserConfigurationException;
import junit.framework.TestCase;
import junit.framework.TestSuite;
import org.w3c.dom.Document;
import javax.xml.XMLConstants;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.*;
import org.xml.sax.SAXException;
import org.junit.Test;

/**
 * Unit test for simple App.
 *
 * Author: Veronika Lovrantova 373905 Date: 20.4.2012
 */
public class AppTest
        extends TestCase {

    DTDCutterImpl cutter = new DTDCutterImpl();
    DTDtoXSDConvertorImpl convertor;

    /**
     * Create the test case
     *
     * @param testName name of the test case
     */
    public AppTest(String testName) {
        super(testName);
    }

    /**
     * @return the suite of tests being tested
     */
    /**
     * Len cisty test funkcnosti Cutter + prevedenia na externom dtd.
     * Predpokladam ze tam ten subor je.
     */
    @Test
    public void testGoodConvertDTD() {
        File xmlFile = null;
        File dtdFile = null;
        File parsedDTD = null;        
        FileInputStream stream = null;
        FileInputStream preparedStream = null;        
        FileOutputStream output = null;
        File schemaFile = null;
        try {
            convertor = new DTDtoXSDConvertorImpl();
            xmlFile = new File("adresar.xml");
            dtdFile = new File("adresar.dtd");
            stream = new FileInputStream(dtdFile);
            parsedDTD = cutter.prepareFile(stream);
            preparedStream = new FileInputStream(parsedDTD);
            cutter.parseDTD(preparedStream);            

            List<Entity> entities = cutter.getEntities();
            List<Attribute> attributes = cutter.getAttributes();
            List<ElementDTD> elements = cutter.getElements();
            List<String> comments = cutter.getComments();


            Document schemaDoc = convertor.convertDTD(elements, attributes, entities, comments);
            Printer printer = new PrinterImpl(schemaDoc);
            schemaFile = new File("testxsd1.xsd");
            output = new FileOutputStream(schemaFile);
            printer.printDocument(schemaDoc, output);


            SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema = schemaFactory.newSchema(new StreamSource(schemaFile));

            Validator validator = schema.newValidator();
            Source xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
            } catch (SAXException ex) {
                fail(ex.getLocalizedMessage());
            }

            xmlFile = new File("adresar_validny.xml");
            xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
            } catch (SAXException ex) {
                fail(ex.getLocalizedMessage());
            }

            xmlFile = new File("adresar_nevalidny1.xml");
            xmlSource = new StreamSource(xmlFile);

            try {
                validator.validate(xmlSource);
                fail("Schema validates nonvalid xml 'adresar_nevalidny1.xml'. Schema is saved in testxsd1.xsd");
            } catch (SAXException ex) {
            }

            xmlFile = new File("adresar_nevalidny2.xml");
            xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
                fail("Schema validates nonvalid xml 'adresar_nevalidny2.xml'. Schema is saved in testxsd1.xsd");
            } catch (SAXException ex) {
            }

        } catch (XSDConvertException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (ParserConfigurationException | FileNotFoundException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (SAXException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } finally {
            try {
                if (stream != null) {
                    stream.close();
                }
                if (preparedStream != null) {
                    preparedStream.close();
                }               
                if (output != null) {
                    output.close();
                }               
                if (parsedDTD != null) {
                    parsedDTD.delete();
                }
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);

            }
        }
    }

    @Test
    public void testGoodConvertDTD2() {
        File xmlFile = null;
        File dtdFile = null;
        File parsedDTD = null;        
        FileInputStream stream = null;
        FileInputStream preparedStream = null;        
        FileOutputStream output = null;
        File schemaFile = null;
        try {
            convertor = new DTDtoXSDConvertorImpl();
            xmlFile = new File("externy.xml");
            dtdFile = new File("externy.dtd");

            stream = new FileInputStream(dtdFile);
            parsedDTD = cutter.prepareFile(stream);
            preparedStream = new FileInputStream(parsedDTD);
            cutter.parseDTD(preparedStream);            

            List<Entity> entities = cutter.getEntities();
            List<Attribute> attributes = cutter.getAttributes();
            List<ElementDTD> elements = cutter.getElements();
            List<String> comments = cutter.getComments();


            Document schemaDoc = convertor.convertDTD(elements, attributes, entities, comments);
            Printer printer = new PrinterImpl(schemaDoc);
            schemaFile = new File("testxsd2.xsd");
            output = new FileOutputStream(schemaFile);
            printer.printDocument(schemaDoc, output);

            SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema = schemaFactory.newSchema(new StreamSource(schemaFile));
            Validator validator = schema.newValidator();
            Source xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
            } catch (SAXException ex) {
                fail(ex.getLocalizedMessage());
            }
            xmlFile = new File("externy_validny.xml");
            xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
            } catch (SAXException ex) {
                fail(ex.getLocalizedMessage());
            }

            xmlFile = new File("externy_nevalidny1.xml");
            xmlSource = new StreamSource(xmlFile);

            try {
                validator.validate(xmlSource);
                fail("Schema validates nonvalid xml 'externy_nevalidny1.xml'. Schema is saved in testxsd2.xsd");
            } catch (SAXException ex) {
            }

            xmlFile = new File("externy_nevalidny2.xml");
            xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
                fail("Schema validates nonvalid xml 'externy_nevalidny2.xml'. Schema is saved in testxsd2.xsd");
            } catch (SAXException ex) {
            }

        } catch (XSDConvertException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (ParserConfigurationException | FileNotFoundException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (SAXException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } finally {
            try {
                if (stream != null) {
                    stream.close();
                }
                if (preparedStream != null) {
                    preparedStream.close();
                }               
                if (output != null) {
                    output.close();
                }                
                if (parsedDTD != null) {
                    parsedDTD.delete();
                }
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            }
        }
    }

    @Test
    public void testGoodConvertDTD3() {
        File xmlFile = null;
        File dtdFile = null;
        File parsedDTD = null;

        FileInputStream stream = null;
        FileInputStream preparedStream = null;
        FileOutputStream output = null;
        File schemaFile = null;
        try {
            convertor = new DTDtoXSDConvertorImpl();
            xmlFile = new File("interny.xml");
            stream = new FileInputStream(xmlFile);
            parsedDTD = cutter.prepareFile(stream);
            preparedStream = new FileInputStream(parsedDTD);
            cutter.parseDTD(preparedStream);

            List<Entity> entities = cutter.getEntities();
            List<Attribute> attributes = cutter.getAttributes();
            List<ElementDTD> elements = cutter.getElements();
            List<String> comments = cutter.getComments();


            Document schemaDoc = convertor.convertDTD(elements, attributes, entities, comments);
            Printer printer = new PrinterImpl(schemaDoc);
            schemaFile = new File("testxsd3.xsd");
            output = new FileOutputStream(schemaFile);
            printer.printDocument(schemaDoc, output);

            SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema = schemaFactory.newSchema(new StreamSource(schemaFile));
            Validator validator = schema.newValidator();
            Source xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
            } catch (SAXException ex) {
                fail(ex.getLocalizedMessage());
            }

            xmlFile = new File("interny_validny.xml");
            xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
            } catch (SAXException ex) {
                fail(ex.getLocalizedMessage());
            }

            xmlFile = new File("interny_nevalidny1.xml");
            xmlSource = new StreamSource(xmlFile);

            try {
                validator.validate(xmlSource);
                fail("Schema validates nonvalid xml 'interny_nevalidny1.xml'.Schema is saved in testxsd3.xsd");
            } catch (SAXException ex) {
            }

            xmlFile = new File("interny_nevalidny2.xml");
            xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
                fail("Schema validates nonvalid xml 'interny_nevalidny1.xml'.Schema is saved in testxsd3.xsd ");
            } catch (SAXException ex) {
            }

        } catch (XSDConvertException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (ParserConfigurationException | FileNotFoundException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (SAXException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } finally {
            try {
                if (stream != null) {
                    stream.close();
                }
                if (preparedStream != null) {
                    preparedStream.close();
                }
                if (output != null) {
                    output.close();
                }
                if (parsedDTD != null) {
                    parsedDTD.delete();
                }
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);

            }
        }
    }
    
    //Doplnenie testov pre dualne DTD (take co je napoli v DTD a napoli v XML)
    //teda funguje tak ako aplikacia cela ale len s tym ze ma na tvrdo dane veci a testuje
    //ich...
    //dualne xml je vytvorene z externy.xml suboru teda pouziva k nemu vytvorene
    //nevalidne xml na testovanie.
    @Test
    public void testGoodDualDTD() {
        File xmlFile = null;
        File dtdFile = null;
        File parsedDTD = null;
        File parsedXML = null;
        FileInputStream xmlStream = null;
        FileInputStream stream = null;
        FileInputStream preparedStream = null;
        FileInputStream preparedStream2 = null;
        FileOutputStream output = null;
        File schemaFile = null;
        try {
            convertor = new DTDtoXSDConvertorImpl();
            xmlFile = new File("dualne.xml");
            dtdFile = new File("dualne.dtd");
            stream = new FileInputStream(dtdFile);
            parsedDTD = cutter.prepareFile(stream);
            preparedStream = new FileInputStream(parsedDTD);
            cutter.parseDTD(preparedStream);

            xmlStream = new FileInputStream(xmlFile);
            parsedXML = cutter.prepareInternFile(xmlStream);
            preparedStream2 = new FileInputStream(parsedXML);
            cutter.parseDTD(preparedStream2);

            List<Entity> entities = cutter.getEntities();
            List<Attribute> attributes = cutter.getAttributes();
            List<ElementDTD> elements = cutter.getElements();
            List<String> comments = cutter.getComments();


            Document schemaDoc = convertor.convertDTD(elements, attributes, entities, comments);
            Printer printer = new PrinterImpl(schemaDoc);
            schemaFile = new File("testxsd4.xsd");
            output = new FileOutputStream(schemaFile);
            printer.printDocument(schemaDoc, output);


            SchemaFactory schemaFactory = SchemaFactory.newInstance(XMLConstants.W3C_XML_SCHEMA_NS_URI);
            Schema schema = schemaFactory.newSchema(new StreamSource(schemaFile));

            Validator validator = schema.newValidator();
            Source xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
            } catch (SAXException ex) {
                fail(ex.getLocalizedMessage());
            }

            xmlFile = new File("externy_validny.xml");
            xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
            } catch (SAXException ex) {
                fail(ex.getLocalizedMessage());
            }

            xmlFile = new File("externy_nevalidny1.xml");
            xmlSource = new StreamSource(xmlFile);

            try {
                validator.validate(xmlSource);
                fail("Schema validates nonvalid xml 'externy_nevalidny1.xml'. Schema is saved in testxsd4.xsd");
            } catch (SAXException ex) {
            }

            xmlFile = new File("externy_nevalidny2.xml");
            xmlSource = new StreamSource(xmlFile);
            try {
                validator.validate(xmlSource);
                fail("Schema validates nonvalid xml 'externy_nevalidny2.xml'. Schema is saved in testxsd4.xsd");
            } catch (SAXException ex) {
            }

        } catch (XSDConvertException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (ParserConfigurationException | FileNotFoundException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (SAXException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } catch (IOException ex) {
            java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);
            fail(ex.getMessage());
        } finally {
            try {
                if (stream != null) {
                    stream.close();
                }
                if (preparedStream != null) {
                    preparedStream.close();
                }
                if (preparedStream2 != null) {
                    preparedStream2.close();
                }
                if (xmlStream != null) {
                    xmlStream.close();
                }
                if (output != null) {
                    output.close();
                }
                if (parsedXML != null) {
                    parsedXML.delete();
                }
                if (parsedDTD != null) {
                    parsedDTD.delete();
                }
            } catch (IOException ex) {
                java.util.logging.Logger.getLogger(AppTest.class.getName()).log(Level.SEVERE, null, ex);

            }
        }
    }
}