package cz.muni.fi.pb138.dtdToXsd;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import org.junit.Before;
import org.junit.Test;
import static org.hamcrest.CoreMatchers.*;
import static org.junit.Assert.*;


/**
 * Created by IntelliJ IDEA.
 * Author: Roman Jakubco 374241
 * Date: 20.4.2012
 * Time: 17:40
 */
public class DTDCutterImplTest {
    
    DTDCutter cutter;
    
    
    @Before
    public void setUp() throws Exception {
        cutter=new DTDCutterImpl();
        

    }

    @Test
    public void testGetElementsFromSource() throws Exception {
        String test1="ELEMENT zaznam ((firma)?, (osoba)?  , (     adresa)*, (email)*, (telefon)*, (web    )*, (icq)*, ((narozen)?), (klicslova)?, (poznamka)?)>";
        String test2="ELEMENT adresa (ulice, ((psc, mesto) | (mesto, psc)), (stat)?, (poznamka)?)>";
        String test3="ELEMENT Tablespaces (Tablespace+)>";
        String test4="ELEMENT adresa ((ulice), ((psc, (mesto)) , ((mesto), (psc))), ((stat)?), (poznamka)?)*>";
        String test5="ELEMENT titul (((#PCDATA)))>";
        String test6="ELEMENT titul ((#PCDATA))>";
        
        ElementDTD elTest1 = elementSetUp("zaznam", "(firma?,osoba?,adresa*,email*,telefon*,web*,icq*,narozen?,klicslova?,poznamka?)");
        ElementDTD elTest2 = elementSetUp("adresa", "(ulice,((psc,mesto)|(mesto,psc)),stat?,poznamka?)");
        ElementDTD elTest3 = elementSetUp("Tablespaces", "Tablespace+");
        ElementDTD elTest4 = elementSetUp("adresa", "(ulice,((psc,mesto),(mesto,psc)),stat?,poznamka?)*");
        ElementDTD elTest5 = elementSetUp("titul", "#PCDATA");
        ElementDTD elTest6 = elementSetUp("titul", "#PCDATA");
        
        assertEquals (cutter.getElementFromSource(test1),elTest1);
        assertEquals (cutter.getElementFromSource(test2),elTest2);
        assertEquals (cutter.getElementFromSource(test3),elTest3);
        assertEquals (cutter.getElementFromSource(test4),elTest4);
        assertEquals (cutter.getElementFromSource(test5),elTest5);
        assertEquals (cutter.getElementFromSource(test6),elTest6);
        
    }

    @Test
    public void testGetAttributesFromSource() throws Exception {
        String test1="ATTLIST adresa typ (soukromá | firemní) \"soukromá\">";
        String test2="ATTLIST firma obor CDATA #IMPLIED>";
        String test3="ATTLIST Datafile  name       CDATA         #REQUIRED>";
        String test4="ATTLIST Datafile  name       CDATA         #FIXED \"500\">";
        String test5="ATTLIST Datafile  name       CDATA        >";
        
        Attribute at1=attributeSetUp("adresa", "typ", "(soukromá|firemní)", "soukromá", null);
        Attribute at2=attributeSetUp("firma", "obor", "CDATA", "#IMPLIED", null);
        Attribute at3=attributeSetUp("Datafile", "name", "CDATA", "#REQUIRED", null);
        Attribute at4=attributeSetUp("Datafile", "name", "CDATA", "#FIXED", "500");
        Attribute at5=attributeSetUp("Datafile", "name", "CDATA", "#IMPLIED", null);
        
        assertEquals (cutter.getAttributeFromSource(test1),at1);
        assertEquals (cutter.getAttributeFromSource(test2),at2);
        assertEquals (cutter.getAttributeFromSource(test3),at3);
        assertEquals (cutter.getAttributeFromSource(test4),at4);
        assertEquals (cutter.getAttributeFromSource(test5),at5);
        
    }

    @Test
    public void testGetEntitiesFromSource() throws Exception {        
        String test1="ENTITY AUTHOR \"Jeffrey Hunter\">";
        String test2="ENTITY WEB    \"www.iDevelopment.info\">";
        String test3="ENTITY EMAIL  \"jhunter@iDevelopment.info\">";
        
        Entity at1=entitySetUp("AUTHOR", "Jeffrey Hunter");
        Entity at2=entitySetUp("WEB", "www.iDevelopment.info");
        Entity at3=entitySetUp("EMAIL", "jhunter@iDevelopment.info");
        
        assertEquals (cutter.getEntityFromSource(test1),at1);
        assertEquals (cutter.getEntityFromSource(test2),at2);
        assertEquals (cutter.getEntityFromSource(test3),at3);

    }
    
    public ElementDTD elementSetUp(String name, String category){
        ElementDTD el=new ElementDTD();
        el.setCategory(category);
        el.setElementName(name);
        return el;
    }
    
    public Attribute attributeSetUp(String elementName,String name, String type,String defaultType,String fixed){
        Attribute at=new Attribute();
        at.setElementName(elementName);
        at.setName(name);
        at.setType(type);
        at.setDefaultType(defaultType);
        at.setFixed(fixed);
        return at;
    }
    
    public Entity entitySetUp(String entityName,String value){
        Entity entity=new Entity();
        entity.setEntityName(entityName);
        entity.setValue(value);
        return entity;
    }
    
}
