package cz.muni.fi.pb138.dtd_to_xsd;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.List;
import java.util.Map;

/**
 * Created by IntelliJ IDEA.
 * Author: Roman Jakubco 374241
 * Date: 20.4.2012
 * Time: 17:32
 */
public interface DTDtoXSDConvertor {
    /**
     * method converts each String representing DTD element into equal XML Schema
     * DOM Element
     *
     *
     * @param inputElements - each String contains information from DTD about one Element
     * @return NodeList containing DOM Elements or empty NodeList if inputElements is null or empty
     * @throws XSDConvertException when some error occures during conversion
     */
    public NodeList convertElements(List<String> inputElements);

    /**
     * converts String representig DTD element into equal XMLSchema DOM Element
     * expected format is valid DTD attribute i.e.
     * <!ELEMENT TVSCHEDULE (CHANNEL+)>
     * @param toConvert
     * @return DOM Element representing XML Schema of given DTD element
     * @throws XSDConvertException when some error occures during conversion
     */
    public Element convertElement(String toConvert);

    /**
     * method converts each String representing DTD attribute into equal XML Schema
     * DOM Element
     *
     * @param inputAttributes - each String contains information from DTD about one attribute
     * @return NodeList containing DOM Elements or empty NodeList if given List is null or empty
     * @throws XSDConvertException when some error occures during conversion
     */
    public NodeList convertAttributes(List<String> inputAttributes);

    /**
     * converts String representig DTD attribute into equal XMLSchema DOM Element
     * expected format is valid DTD attribute i.e.
     * <!ATTLIST TVSCHEDULE NAME CDATA #REQUIRED>
     * @param toConvert
     * @return DOM Element representing XML Schema of given DTD Element
     * @throws XSDConvertException when some error occures during conversion
     */
    public Element convertAttribute(String toConvert);


    /**
     * method converts each String representing DTD entity into equal XML Schema
     * DOM Element
     *
     * @param inputEntities - each String contains information from DTD about one entity
     * @return NodeList containing DOM Elements or empty NodeList if given List is null or empty
     * @throws XSDConvertException when some error occures during conversion
     */
    public NodeList convertEntities(List<String> inputEntities);

    /**
     * converts String representig DTD entity into equal XMLSchema DOM Element
     * expected format is valid DTD entity i.e.
     * <!ENTITY entity-name SYSTEM "URI/URL">
     *
     * @param toConvert
     * @return DOM Element representing XML Schema of given DTD entity
     * @throws XSDConvertException when some error occures during conversion
     */
    public Element convertEntity(String toConvert);

    /**
     * converts given DTD doctype into equlal XML Schema DOM Element
     * expected format is DTD doctype i.e.
     * <!DOCTYPE NEWSPAPER>
     * @param toConvert
     * @return DOM Element representing XMLSchmeda of given DTD doctype
     */
    public Element convertDoctype(String toConvert);

    /**
     * method converts whole DTD document splitted into four parts (elements,attributes,
     * entities,doctype) each part is also key value of given Map;
     *
     * @param DTDtoConvert
     * @return NodeList containing whole DTD document converted into XMLSchmema
     */

    public NodeList convertDTD(Map<String,List<String>> DTDtoConvert);

}
